////////////////////////////////////////////////////////////////////////////////
///
///		Title		: Abstract Button File
///
///		Copyright (c) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: Desktop Service Development
///
////////////////////////////////////////////////////////////////////////////////
#pragma once

#include "ActionEvent.h"
#include "ImageProxy.h"
#include "NButtonInterface.h"
#include "NControl.h"
#include "NToolTip.h"
#include "StateControl.h"

//properties
class AbstractButtonProperty
{
public:
	static const TCHAR* Pushed;
};
/**
* @ingroup 	Base UI	
* @brief 	abstract button class
* @date		10/25/2006
* @author	telltale
* @remark	
*/
class NPCOMMONUI_API CAbstractButton : public CNControl, public IStatable
{
	DECLARE_RTTI;
public:
	/*{kr}
	ư Ÿ .
	⺻ Ÿ ư.
	Push Ÿ ư
	{/kr}*/
	enum eButtonStyle
	{
		E_BUTTON_STYLE_STAND,
		E_BUTTON_STYLE_PUSHED,
	};

	/*{kr}
	ư .
	None .
	Selected .
	Over .
	{/kr}*/
	enum eButtonState
	{
		E_BUTTON_STATE_NONE,
		E_BUTTON_STATE_SELECTED,
		E_BUTTON_STATE_OVER,		
	};

	CAbstractButton(void);
	virtual ~CAbstractButton(void);

	void SetStyle(eButtonStyle eStyle);
	eButtonStyle GetStyle();
	/**
	 * @breif {kr}  ư ° Pushed  {/kr}
	 */
	BOOL GetPushed();
	void SetPushed(BOOL bPushed);
	void SetToolTipText(LPCTSTR szToolTipText);
	/**
	 * \brief {kr} State ´. {/kr}
	 * \return {kr}  State {/kr}
	 */
	eButtonState GetState();
	/**
	* @brief 	add the specified action listener to receive action event from this button
	* @param	pActionListener	action listener
	* @return	none
	*/
	void AddActionListener(IActionListener* pActionListener);

	/**
	* @brief 	remove the specified action listener to receive action event from this button
	* @param	pActionListener	action listener
	* @return	none
	*/
	void RemoveActionListener(IActionListener* pActionListner);
	void SetBtnStateBgDg(IButtonStateChangeDelegate* pBtnStateBgDg);
	/**
	 * \breif {kr} 콺 ư Ŭ  ȿ Ÿ. {/kr}
	 */
	void DoClick();

	//override
	//CComponent
	BOOL EnableWindow(BOOL bEnabled);
	void OnShowNotify();
	void OnHideNotify();
	
	//message loop
	BEGIN_MSG_MAP(CAbstractButton)
		MESSAGE_HANDLER(UM_INVOKELATER_SHOWTOOLTIP, _OnInvokeLaterShowToolTip)

		CHAIN_MSG_MAP(CNControl)
	END_MSG_MAP()

protected:
	//override
	//CComponent
	void OnSize(INT32 nWidth, INT32 nHeight);
	void OnLButtonDblClk(UINT nFlags, CPoint point);
	void OnLButtonDown(UINT nFlags, CPoint point);
	void OnLButtonUp(UINT nFlags, CPoint point);
	void OnMouseMove(UINT nFlags, CPoint point);
	void OnMouseEnter(CPoint point);
	void OnMouseLeave();
	
	//virtual
	virtual void FireAction();

	//message handler
	LRESULT _OnInvokeLaterShowToolTip(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	
private:
	//mouse button down
	void _OnLBtnDown();
	void _OnLBtnUp();
	
	BOOL m_bEnableToolTip; ///< Has Button Tooltip?
	BOOL m_bPushed; ///< Is Pushed State?
	CAtlList<IActionListener*>	m_vActionListener;	///< actionlistener list
	CButtonStateCgDg_Def m_BtnStateCgDg_Def;
	CNToolTip m_ToolTip; ///< Button Tool Tip Ctrl
	CString	m_strText; ///< button title
	IButtonStateChangeDelegate* m_pBtnStateCgDg;
	///< {kr} n  state, n bitmap   ִ. states total number
	///< {kr} [Normal][Over][Selected][Dis][Pushed] {/kr}
	int m_nCount;
	eButtonState m_eState; ///< button state
	eButtonStyle m_eStyle; ///< button style
	BOOL m_bMouseDown;

	static const int CAbstractButton::Prop_ToolTipOffsetX = 0;
	static const int CAbstractButton::Prop_ToolTipOffsetY = 0;

	static const UINT UM_INVOKELATER_SHOWTOOLTIP = WM_USER + 1;
};